@@ -1,3 +1,6 @@
+{-# LANGUAGE MultiParamTypeClasses, FlexibleInstances
+, FlexibleContexts, UndecidableInstances #-}
+
 -----------------------------------------------------------
 -- |
 -- Module      :  BoundedList
