@@ -22,7 +22,7 @@ module Database.HaskellDB.DBSpec.DBInfo
 
 import qualified Database.HaskellDB.DBSpec.PPHelpers as PP
 import Database.HaskellDB.FieldType (FieldDesc, FieldType(BStrT, StringT), )
-import Data.Char (toLower, )
+import Data.Char (toLower, isAlpha)
 import Text.PrettyPrint.HughesPJ
 
 -- | Defines a database layout, top level
@@ -57,10 +57,11 @@ instance Show DBOptions where
 -- | Creates a valid declaration of a DBInfo. The variable name will be the
 --   same as the database name
 dbInfoToDoc :: DBInfo -> Doc
-dbInfoToDoc dbi@(DBInfo {dbname=n}) 
+dbInfoToDoc dbi@(DBInfo {dbname = n
+                        , opts = opt}) 
     = fixedName <+> text ":: DBInfo"
       $$ fixedName <+> equals <+> ppDBInfo dbi
-      where fixedName = text ((toLower . head) n : tail n)
+      where fixedName = text . PP.identifier (makeIdent opt) $ n
 
 -- | Pretty prints a DBInfo
 ppDBInfo :: DBInfo -> Doc
