@@ -18,14 +18,14 @@
 module Database.HaskellDB.Optimize (optimize, optimizeCriteria) where
 
 import Control.Exception (assert)
-import Data.List (intersect,(\\),union)
+import Data.List (intersect, (\\), union, nub)
 import Database.HaskellDB.PrimQuery
 
 -- | Optimize a PrimQuery
 optimize :: PrimQuery -> PrimQuery
 optimize = hacks
            . mergeProject
-	   . removeEmpty
+           . removeEmpty
            . removeDead
            . pushRestrict
            . optimizeExprs
@@ -82,36 +82,7 @@ removeD live (Project assoc query)
 	  -- or that will be put in a GROUP BY clause.
 	  -- These are the associations that will be kept.
 	  newAssoc :: Assoc
-          newAssoc      | hasAggregate  = groupAssoc ++ liveAssoc
-          		| otherwise	= liveAssoc
-          		where
-          		  -- when an aggregate expression is in the
-          		  -- association we check
-          		  -- if an attribute is explicitly added by the user
-			  -- and not already live
-          		  -- (ie. "extend" is called), if so we should
-          		  -- keep it live to be added in a GROUP BY
-          		  -- clause.
-          		  groupAssoc	    = filter (not.isLive)
-          		  		    $ filter newAttr assoc
-			  -- Does the association define a new attribute?
-			  -- (i.e. not just pass on an existing one
-			  -- from the nested query)
-			  newAttr :: (Attribute,PrimExpr) -> Bool
-          		  newAttr (attr,AttrExpr name)  = (attr /= name)
-          		  newAttr _   		  	= True
-
-          -- Is any live attribute bound to an aggregate expression?
-	  hasAggregate :: Bool
-	  hasAggregate  = any (isAggregate.snd) liveAssoc
-
-	  -- All associations that define live attributes.
-	  liveAssoc :: Assoc
-          liveAssoc          = filter isLive assoc
-
-	  -- Is the attribute defined by the association live?
-	  isLive :: (Attribute,PrimExpr) -> Bool
-          isLive (attr,expr) = attr `elem` live
+          newAssoc = filter (isLive live) assoc
 
 removeD live (Restrict x query)
         = Restrict x (removeD (live ++ attrInExpr x) query)
@@ -119,27 +90,32 @@ removeD live (Restrict x query)
 removeD live (Special (Order xs) query)
 	= Special (Order xs) (removeD (live ++ attrInOrder xs) query)
 
--- Filter dead columns from group expression, as they are not used. Note
--- live columns are NOT just those that are in the select, but also those
--- used in restrictions.
-removeD live (Group cols query)
-    = Group liveCols (removeD (live ++ (map fst liveCols)) query)
+removeD live (Group cols query) = Group newAssoc (removeD newLive query)
   where
-    liveCols = filter ((`elem` live) . fst) cols
-  
-removeD live query
-        = query
+    newLive :: Scheme
+    newLive = concat (map (attrInExpr . snd) newAssoc)
+    newAssoc :: Assoc
+    newAssoc = filter (isLive live) cols
 
+removeD live query = query
+
+-- | Determines if the given column (attribute/expression pair)
+-- exists in the scheme given.
+isLive :: Scheme -> (Attribute,PrimExpr) -> Bool
+isLive live (attr,expr) = attr `elem` live
+
+schemeOf :: (PrimExpr -> Bool) -> Assoc -> Scheme
+schemeOf f = map fst . filter (f . snd)
 
 -- | Remove unused parts of the query
 removeEmpty :: PrimQuery -> PrimQuery
-removeEmpty
+removeEmpty 
         = foldPrimQuery (Empty, BaseTable, project, restrict, binary, group, special)
         where
           -- Messes up queries without a table, e.g. constant queries
 	  -- disabled by Bjorn Bringert 2004-04-08
           --project assoc Empty   = Empty
-          project assoc query   | null assoc    = Empty
+          project assoc query   | null assoc    = query
                                 | otherwise     = Project assoc query
 
           restrict x Empty      = Empty
@@ -158,43 +134,64 @@ removeEmpty
           group _ Empty = Empty
           group cols query = Group cols query
 
-
 -- | Collapse adjacent projections
 mergeProject :: PrimQuery -> PrimQuery
-mergeProject
-        = foldPrimQuery (Empty,BaseTable,project,Restrict,Binary,Group, Special)
+mergeProject q
+        = foldPrimQuery (Empty, BaseTable, project, Restrict, Binary, Group, Special) q
         where
           project assoc1 (Project assoc2 query)
-             	| safe newAssoc	  = Project newAssoc query
+                | equal assoc1 assoc2 = Project assoc2 query
+             	| safe assoc1 query = Project (subst assoc1 assoc2) query
              	where
-             	  newAssoc = subst assoc1 assoc2
 
-	  -- "hmm, is this always true ?" (Daan Leijen)
-	  -- "no, not true when assoc uses fields defined in only
-	  -- one of assoc1 or assoc2, which happens
-	  -- when op == Times" (Bjorn Bringert)
 	  project assoc query@(Binary Times _ _) = Project assoc query
 	  project assoc (Binary op (Project assoc1 query1)
           		           (Project assoc2 query2))
-          	| safe newAssoc1 && safe newAssoc2
+          	| safe assoc1 query1 && safe assoc2 query2
           		= Binary op (Project newAssoc1 query1)
           		            (Project newAssoc2 query2)
           		where
           		  newAssoc1  = subst assoc assoc1
           		  newAssoc2  = subst assoc assoc2
-
+          
           project assoc query
                 = Project assoc query
 	 
+          -- Replace columns in a1 with
+          -- expressions from a2. 
 	  subst :: Assoc -- ^ Association that we want to change
 		-> Assoc -- ^ Association containing the substitutions
 		-> Assoc
           subst a1 a2
                 = map (\(attr,expr) -> (attr, substAttr a2 expr)) a1
 
-          safe :: Assoc -> Bool
-          safe assoc
-          	= not (any (isAggregate.snd) assoc)
+          -- It is safe to merge two projections in two cases. 
+          --  1. All columns in the outer projections are attributes, with no
+          --     computation. This means they merely copy values from the inner
+          --     projection to the outer and can be eliminated.
+          --  2. The inner projection does not group on any columns.
+          safe :: Assoc -- ^ Outer projection's columns.
+               -> PrimQuery -- ^ Inner projection's query.
+               -> Bool
+          safe outer innerQuery = all isAttr outer || null (groups innerQuery)
+            where
+              isAttr (_, AttrExpr _) = True
+              isAttr _ = False
+          
+          -- Are two associations equal?
+          equal :: Assoc -> Assoc -> Bool
+          equal assoc1 assoc2 = length assoc1 == length assoc2 &&
+                                (all (\((a1, _),(a2, _)) -> a1 == a2) $ zip assoc1 assoc2)
+          -- Returns grouped columns for a projection. 
+          groups :: PrimQuery -> Scheme
+          groups = foldPrimQuery ([], \ _ _ -> [],
+                                  \ _ _ -> [], restrict, 
+                                  \ _ _ _ -> [], group, special)
+            where
+              restrict _ rest = rest
+              group cols _ = map fst cols
+              special _ rest = rest
+
 
 -- | Push restrictions down through projections and binary ops.
 pushRestrict :: PrimQuery -> PrimQuery
@@ -205,6 +202,9 @@ pushRestrict (Binary op query1 query2)
 pushRestrict (Project assoc query)
         = Project assoc (pushRestrict query)
 
+pushRestrict (Group assoc query)
+        = Group assoc (pushRestrict query)
+
 -- restricts
 
 pushRestrict (Restrict x (Project assoc query))
@@ -214,7 +214,14 @@ pushRestrict (Restrict x (Project assoc query))
 	  -- with the expression they are bound to by the project
           expr  = substAttr assoc x
 	  -- aggregate expressions are not allowed in restricts
-	  safe = not (isAggregate expr)
+	  safe = not (isAggregate expr) && not (hasAggregates query)
+          hasAggregates (Project _ _) = False
+          hasAggregates (BaseTable  _ _)  = False
+          hasAggregates (Restrict _ qry) = hasAggregates qry
+          hasAggregates (Group _ _) = True
+          hasAggregates (Binary _ left right) = hasAggregates left || hasAggregates right
+          hasAggregates (Special   _ qry) = hasAggregates qry
+          hasAggregates Empty = False
 
 pushRestrict (Restrict x (Binary op query1 query2))
         | noneIn1   = Binary op query1 (pushRestrict (Restrict x query2))
@@ -278,7 +285,7 @@ optimizeExprs = foldPrimQuery (Empty, BaseTable, Project, restr, Binary, Group,
           where e' = optimizeExpr e
 
 optimizeExpr :: PrimExpr -> PrimExpr
-optimizeExpr = foldPrimExpr (AttrExpr,ConstExpr,bin,un,AggrExpr,CaseExpr,ListExpr)
+optimizeExpr = foldPrimExpr (AttrExpr,ConstExpr,bin,un,AggrExpr,CaseExpr,ListExpr,ParamExpr,FunExpr, CastExpr)
     where
       bin OpAnd e1 e2
           | exprIsFalse e1 || exprIsFalse e2 = exprFalse
