@@ -1,3 +1,6 @@
+{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances
+  , FlexibleContexts, UndecidableInstances
+  , TypeSynonymInstances #-}
 -----------------------------------------------------------
 -- |
 -- Module      :  Query
@@ -17,35 +20,37 @@
 -----------------------------------------------------------
 module Database.HaskellDB.Query (
 	      -- * Data and class declarations
-	      Rel(..), Attr(..), Table(..), Query, Expr(..), OrderExpr
-	     , ToPrimExprs, ShowConstant
-	     , ExprC, ProjectExpr, ProjectRec, InsertRec
-             , ConstantRecord(..)
+	     Rel(..), Attr(..), Table(..), Query, Expr(..), OrderExpr
+	     , ToPrimExprs, ConstantRecord
+	     , ShowConstant(..), ExprC(..), ProjectExpr, ProjectRec, InsertRec
+	     , ExprAggr(..), ExprDefault(..)
+	     , copy, copyAll, RelToRec
 	      -- * Operators
 	     , (.==.) , (.<>.), (.<.), (.<=.), (.>.), (.>=.)
 	     , (.&&.) , (.||.)
-	     , (.*.) , (./.), (.%.), (.+.), (.-.), (.++.)
+	     , (.*.), (./.), (.+.), (.-.), (.%.), (.++.)
              , (<<), (<<-)
 	      -- * Function declarations
 	     , project, restrict, table, unique
 	     , union, intersect, divide, minus
 	     , _not, like, _in, cat, _length
 	     , isNull, notNull
-	     , fromNull
-	     , constant, constJust
-	     , count, _sum, _max, _min, avg
+	     , fromNull, fromVal
+	     , constant, constVal, constNull, constExpr
+	     , param, namedParam, Args, func, cast
+	     , toStr, coerce , select
+	     , count, _sum, _max, _min, avg , literal
 	     , stddev, stddevP, variance, varianceP
-	     , asc, desc, order
-	     , top --, topPercent
-             , _case
-	     , _default
-             -- * Internals
-	     , runQuery, runQueryRel
-	     , attribute, tableName, baseTable
-	     , attributeName, exprs, labels
+	     , asc, desc, order , top
+	     , _case , _default
+	     -- * Internals
+	     , runQuery, runQueryRel, unQuery
+	     , subQuery
+	     , attribute, attributeName, tableName, baseTable, emptyTable
+	     , exprs, labels, tableRec 
+	     , constantRecord
 	     ) where
-
-import Database.HaskellDB.HDBRec
+import Database.HaskellDB.HDBRec 
 import Database.HaskellDB.PrimQuery
 import Database.HaskellDB.BoundedString
 import Database.HaskellDB.BoundedList
@@ -56,7 +61,6 @@ import System.Time (CalendarTime)
 -- Operators
 -----------------------------------------------------------
 
---infix   9 !
 infix   8 `like`, `_in`
 infixl  7 .*., ./., .%.
 infixl  6 .+.,.-.
@@ -76,14 +80,14 @@ infixr  2 .||.
 data Rel r      = Rel Alias Scheme
 
 -- | Type of normal expressions, contains the untyped PrimExpr.
-data Expr a     = Expr PrimExpr
+newtype Expr a     = Expr PrimExpr
 		deriving (Read, Show)
 
 -- | Type of aggregate expressions.
-data ExprAggr a = ExprAggr PrimExpr deriving (Read, Show)
+newtype ExprAggr a = ExprAggr PrimExpr deriving (Read, Show)
 
 -- | The type of default expressions.
-data ExprDefault a = ExprDefault PrimExpr deriving (Read, Show)
+newtype ExprDefault a = ExprDefault PrimExpr deriving (Read, Show)
 
 -- | Basic tables, contains table name and an
 --   association from attributes to attribute
@@ -131,7 +135,7 @@ instance InsertExpr ExprDefault
 class InsertRec r er | r -> er
 instance InsertRec RecNil RecNil
 instance (InsertExpr e, InsertRec r er) => 
-    InsertRec (RecCons f (e a) r) (RecCons f (Expr a) er)
+  InsertRec (RecCons f (e a) r) (RecCons f (Expr a) er)
 
 -- | Class of expressions that can be used with 'project'.
 class ExprC e => ProjectExpr e
@@ -144,7 +148,7 @@ instance ProjectExpr ExprAggr
 class ProjectRec r er | r -> er
 instance ProjectRec RecNil RecNil
 instance (ProjectExpr e, ProjectRec r er) => 
-    ProjectRec (RecCons f (e a) r) (RecCons f (Expr a) er)
+  ProjectRec (RecCons f (e a) r) (RecCons f (Expr a) er)
 
 -----------------------------------------------------------
 -- Record operators
@@ -160,15 +164,55 @@ _ << x = RecCons x
 -- | Convenience operator for constructing records of constants.
 --   Useful primarily with 'insert'.
 --   @f <<- x@ is the same as @f << constant x@
-( <<- ) :: ShowConstant a => 
-	   Attr f a        -- ^ Field label
-	-> a                        -- ^ Field value
-	-> Record (RecCons f (Expr a) RecNil)  -- ^ New record
+( <<- ) :: ShowConstant a =>
+         Attr f a        -- ^ Field label
+      -> a                        -- ^ Field value
+      -> Record (RecCons f (Expr a) RecNil)  -- ^ New record
 f <<- x = f << constant x
 
------------------------------------------------------------
--- Basic relational operators
------------------------------------------------------------
+-- | Creates a single-field record from an attribute and a table. Useful
+-- for building projections that will re-use the same attribute name. @copy attr tbl@ is
+-- equivalent to:
+--
+--   @attr .=. (tbl .!. attr)@
+--
+copy :: (HasField f r) => Attr f a -> Rel r -> Record (RecCons f (Expr a) RecNil)
+copy attr tbl = attr << tbl ! attr
+
+-- | Copies all columns in the relation given. Useful for appending
+-- the remaining columns in a table to a projection. For example:
+--
+-- >   query = do
+-- >     tbl <- table some_table
+-- >     project $ copyAll tbl
+--
+-- will add all columns in "some_table" to the query.
+copyAll :: (RelToRec r) => Rel r -> Record r
+copyAll = relToRec
+
+-- | Helper class which gives a polymorphic
+-- copy function that can turn a Rel into a Record.
+class RelToRec a where
+  relToRec :: Rel a -> Record a
+
+instance RelToRec RecNil where
+  relToRec v = \_ -> unRel v
+    where 
+      unRel :: Rel r -> r
+      unRel = error "unRel RelToRec RecNil"
+
+-- All this type magic takes the first field off the Rel (Record ...) type, 
+-- turns it into a (Record ...) type, and prepends it to the rest of the 
+-- converted record. 
+instance (RelToRec rest, FieldTag f) => RelToRec (RecCons f (Expr a) rest) where
+  relToRec t@(Rel v s) = copy (attr . fieldT $ t) t # relToRec (restT t) 
+    where
+      attr :: FieldTag f => f -> Attr f a
+      attr = Attr . fieldName
+      fieldT :: Rel (RecCons f a rest) -> f 
+      fieldT = error "fieldT"
+      restT :: Rel (RecCons f a rest) -> Rel rest
+      restT _ = Rel v s
 
 -- | Field selection operator. It is overloaded to work for both
 --   relations in a query and the result of a query.
@@ -183,6 +227,10 @@ select :: HasField f r => Attr f a -> Rel r -> Expr a
 select (Attr attribute) (Rel alias scheme)
         = Expr (AttrExpr (fresh alias attribute))
 
+-----------------------------------------------------------
+-- Basic relational operators
+-----------------------------------------------------------
+
 -- | Specifies a subset of the columns in the table.
 project :: (ShowLabels r, ToPrimExprs r, ProjectRec r er) => Record r -> Query (Rel er)
 project r
@@ -193,35 +241,40 @@ project r
 	  updatePrimQuery (extend assoc)
           return (Rel alias scheme)
 
-
 -- | Restricts the records to only those who evaluates the 
 -- expression to True.
 restrict :: Expr Bool -> Query ()
 restrict (Expr primExpr) = updatePrimQuery_ (Restrict primExpr)
 
 -- | Restricts the relation given to only return unique records. Upshot
--- is all projected attributes will be 'grouped'
+-- is all projected attributes will be 'grouped'.
 unique :: Query ()
 unique = Query (\(i, primQ) ->
     -- Add all non-aggregate expressions in the query
     -- to a groupby association list. This list holds the name
     -- of the expression and the expression itself. Those expressions
     -- will later by added to the groupby list in the SqlSelect built.
-    case findNonAggr primQ of
+    case nonAggr primQ of
       [] -> ((), (i + 1, primQ)) -- No non-aggregate expressions - no-op.
       newCols -> ((), (i + 1, Group newCols primQ)))
   where
-    -- Find all non-aggregate expressions.
-    findNonAggr :: PrimQuery -> Assoc
-    findNonAggr (Project cols q) = filter (not . isAggregate . snd) cols
-    findNonAggr (Restrict _ q) = findNonAggr q
-    findNonAggr (Binary _ q1 q2) = findNonAggr q1 ++ findNonAggr q2
-    findNonAggr (BaseTable tblName cols) = zip cols (map AttrExpr cols)
-    findNonAggr (Special _ q) = findNonAggr q
+    -- Find all non-aggregate expressions and convert
+    -- them to attribute expressions for use in group by.
+    nonAggr :: PrimQuery -> Assoc
+    nonAggr p = map toAttrExpr . filter (not . isAggregate . snd) . projected $ p 
+    toAttrExpr (col, _) = (col, AttrExpr col)
+    -- Find all projected columns from subqueries.
+    projected :: PrimQuery -> Assoc
+    projected (Project cols q) = cols
+    projected (Restrict _ q) = projected q
+    projected (Binary _ q1 q2) = projected q1 ++ projected q2
+    projected (BaseTable tblName cols) = zip cols (map AttrExpr cols)
+    projected (Special _ q) = projected q
     -- Group and Empty are no-ops
-    findNonAggr (Group _ _) = []
-    findNonAggr Empty  = []
-  
+    projected (Group _ _) = []
+    projected Empty  = []
+
+
 -----------------------------------------------------------
 -- Binary operations
 -----------------------------------------------------------
@@ -230,9 +283,8 @@ binrel :: RelOp -> Query (Rel r) -> Query (Rel r) -> Query (Rel r)
 binrel op (Query q1) (Query q2)
   = Query (\(i,primQ) ->
       let (Rel a1 scheme1,(j,primQ1)) = q1 (i,primQ)
-          (Rel a2 scheme2,(k,primQ2)) = q2 (j,primQ)
-
-          alias	  = k
+          (Rel a2 scheme2,(alias,primQ2)) = q2 (j,primQ)
+          
           scheme  = scheme1
 
           assoc1  = zip (map (fresh alias) scheme1)
@@ -244,7 +296,7 @@ binrel op (Query q1) (Query q2)
           r2      = Project assoc2 primQ2
           r       = Binary op r1 r2
       in
-          (Rel alias scheme,(k+1,times r primQ)) )
+          (Rel alias scheme,(alias + 1, times r primQ)) )
 
 -- | Return all records which are present in at least
 --   one of the relations.
@@ -283,19 +335,37 @@ table (Table name assoc)
 tableName :: Table t -> TableName
 tableName (Table n _) = n
 
--- used in table definitions
+-- Type-level function to return the type of a table's row.
+tableRec :: Table (Record r) -> Record r
+tableRec = error "tableRec should never be evaluated."
 
+-- used in table definitions
 baseTable :: (ShowLabels r, ToPrimExprs r) => TableName -> Record r -> Table r
 baseTable t r   = Table t (zip (labels r) (exprs r))
 
+-- | For queries against fake tables, such as
+-- 'information_schema.information_schema_catalog_name'. Useful for
+-- constructing queries that contain constant data (and do not select
+-- from columns) but need a table to select from.
+emptyTable :: TableName -> Table (Record RecNil)
+emptyTable t = Table t []
 
 attribute :: String -> Expr a
 attribute name  = Expr (AttrExpr name)
 
-
 -----------------------------------------------------------
 -- Expressions
 -----------------------------------------------------------
+-- | Create a named parameter with a default value.
+namedParam :: Name -- ^ Name of the parameter.
+  -> Expr a -- ^ Default value for the parameter.
+  -> Expr a 
+namedParam n (Expr def) = Expr (ParamExpr (Just n) def) 
+
+-- | Create an anonymous parameter with a default value.
+param :: Expr a -- ^ Default value.
+  -> Expr a
+param (Expr def) = Expr (ParamExpr Nothing def) 
 
 unop :: UnOp -> Expr a -> Expr b
 unop op (Expr primExpr)
@@ -305,35 +375,23 @@ binop :: BinOp -> Expr a  -> Expr b -> Expr c
 binop op (Expr primExpr1) (Expr primExpr2)
                 = Expr (BinExpr op primExpr1 primExpr2)
 
--- | (.==.) is used in a similar way as the standard op (==) in
--- Haskell and = in SQL, but takes two 'Expr' as arguments and 
--- returns an 'Expr' Bool.
+-- | Equality comparison on Exprs, = in SQL.
 (.==.) :: Eq a => Expr a -> Expr a -> Expr Bool
 (.==.) = binop OpEq
 
--- | (.\<>.) is used in a similar way as the standard op (\/=) in
--- Haskell and \<> in SQL, but takes two 'Expr' as arguments and 
--- returns an 'Expr' Bool.
+-- | Inequality on Exprs, <> in SQL.
 (.<>.) :: Eq a => Expr a -> Expr a -> Expr Bool
 (.<>.) = binop OpNotEq
 
--- | As with (.==.) and (.\<>.), this op has a standard Haskell
--- op counterpart; (\<) and an SQL counterpart; \<
 (.<.) :: Ord a => Expr a -> Expr a -> Expr Bool
 (.<.)  = binop OpLt
 
--- | As with (.==.) and (.\<>.), this op have a standard Haskell
--- op counterpart, (\<=) and an SQL counterpart; <=.
 (.<=.) :: Ord a => Expr a -> Expr a -> Expr Bool
 (.<=.) = binop OpLtEq
 
--- | As with (.==.) and (.\<>.), this op have a standard Haskell
--- op counterpart, (>) and an SQL counterpart; >.
 (.>.) :: Ord a => Expr a -> Expr a -> Expr Bool
 (.>.)  = binop OpGt
 
--- | As with (.==.) and (.\<>.), this op have a standard Haskell
--- op counterpart, (>=) and an SQL counterpart; >=.
 (.>=.) :: Ord a => Expr a -> Expr a -> Expr Bool
 (.>=.) = binop OpGtEq
 
@@ -341,13 +399,11 @@ binop op (Expr primExpr1) (Expr primExpr2)
 _not :: Expr Bool -> Expr Bool
 _not   = unop OpNot
 
--- | \"Logical and\" on 'Expr', similar to the (&&) op in
--- Haskell and AND in SQL.
+-- | \"Logical and\" on 'Expr', AND in SQL.
 (.&&.):: Expr Bool -> Expr Bool -> Expr Bool
 (.&&.) = binop OpAnd
 
--- | \"Logical or\" on 'Expr', similar to the (||) op in
--- Haskell and OR in SQL.
+-- | \"Logical or\" on 'Expr'. OR in SQL.
 (.||.) :: Expr Bool -> Expr Bool -> Expr Bool
 (.||.) = binop OpOr
 
@@ -391,7 +447,6 @@ _length = unop OpLength
 numop :: Num a => BinOp -> Expr a -> Expr a -> Expr a
 numop   = binop
 
-
 -- | Addition
 (.+.) :: Num a => Expr a -> Expr a -> Expr a
 (.+.) = numop OpPlus
@@ -426,13 +481,101 @@ _case :: [(Expr Bool, Expr a)] -- ^ A list of conditions and expressions.
       -> Expr a
 _case cs (Expr el) = Expr (CaseExpr [ (c,e) | (Expr c, Expr e) <- cs] el)
 
--- | Takes a default value a and a nullable value. If the value is NULL,
---   the default value is returned, otherwise the value itself is returned.
---   Simliar to 'fromMaybe'
-fromNull :: Expr a         -- ^ Default value (to be returned for 'Nothing')
-	 -> Expr (Maybe a) -- ^ A nullable expression
-	 -> Expr a
-fromNull d x@(Expr px) = _case [(isNull x, d)] (Expr px)
+-- | Class which can convert BoundedStrings to normal strings,
+-- even inside type constructors. Useful when a field
+-- is defined as a BoundedString (e.g. "Expr BStr10" or "Expr (Maybe BStr20)") but
+-- it needs to be used in an expression context. The example below illustrates a
+-- table with at least two fields, strField and bStrField. The first is defined as
+-- containing strings, the second as containing strings up to 10 characters long. The
+-- @toStr@ function must be used to convert the bStrField into the appropriate type for
+-- projecting as the strField:
+--
+-- > type SomeTable = (RecCons StrField (Expr String)
+-- >                    (RecCons BStrField (Expr BStr10) ... ))
+--
+-- > someTable :: Table SomeTable
+-- > someTable = ...
+--
+-- > strField :: Attr StrField String
+-- > strField = ...
+-- >
+-- > bstrField :: Attr BStrField (BStr10)
+-- > bstrField = ...
+-- > 
+-- > query = do
+-- >  t <- table someTable
+-- >  project $ strField << toStr $ t ! bstrField
+--
+class BStrToStr s d where
+  -- | Convert a bounded string to a real string.
+  toStr :: s -> d
+
+instance (Size n) => BStrToStr (Expr (BoundedString n)) (Expr String) where
+  toStr (Expr e) = (Expr e)
+
+instance (Size n) => BStrToStr (Expr (Maybe (BoundedString n))) (Expr (Maybe String)) where
+  toStr (Expr m) = (Expr m)
+
+instance BStrToStr (Expr (Maybe String)) (Expr (Maybe String)) where
+  toStr (Expr m) = (Expr m)
+
+instance BStrToStr (Expr String) (Expr String) where
+  toStr (Expr m) = (Expr m)
+
+-----------------------------------------------------------
+-- Using arbitrary SQL functions in a type-safe way.
+-----------------------------------------------------------
+
+-- | Used to implement variable length arguments to @func@, below.
+class Args a where
+  arg_ :: String -> [PrimExpr] -> a
+
+-- | Used to limit variable argument form of @func@ to only take @Expr@ types,
+-- and ignore @ExprAggr@ types.
+class IsExpr a
+
+instance (IsExpr tail) => IsExpr (Expr a -> tail)
+
+instance IsExpr (Expr a)
+
+instance (IsExpr tail, Args tail) => Args (Expr a -> tail) where
+  arg_ name exprs = \(Expr prim) -> arg_ name (prim : exprs)
+
+instance Args (Expr a) where
+  -- Reverse necessary because arguments are built in reverse order by instances
+  -- of Args above.
+  arg_ name exprs = Expr (FunExpr name (reverse exprs))
+
+instance Args (Expr a -> ExprAggr c) where
+  arg_ name exprs = \(Expr prim) -> ExprAggr (AggrExpr (AggrOther name) prim)
+
+{- | Can be used to define SQL functions which will
+appear in queries. Each argument for the function is specified by its own Expr value. 
+Examples include:
+
+>  lower :: Expr a -> Expr (Maybe String) 
+>  lower str = func "lower" str
+
+The arguments to the function do not have to be Expr if they can
+be converted to Expr:
+
+>  data DatePart = Day | Century deriving Show 
+
+>  datePart :: DatePart -> Expr (Maybe CalendarTime) -> Expr (Maybe Int) 
+>  datePart date col = func "date_part" (constant $ show date) col
+
+Aggregate functions can also be defined. For example:
+
+ >  every :: Expr Bool -> ExprAggr Bool 
+ >  every col = func "every" col
+
+Aggregates are implemented to always take one argument, so any attempt to
+define an aggregate with any more or less arguments will result in an error.
+
+Note that type signatures are usually required for each function defined,
+unless the arguments can be inferred.-}
+func :: (Args a) => String -> a 
+func name = arg_ name [] 
 
 -----------------------------------------------------------
 -- Default values
@@ -477,11 +620,56 @@ instance Size n => ShowConstant (BoundedString n) where
 constant :: ShowConstant a => a -> Expr a
 constant x  = Expr (ConstExpr (showConstant x))
 
+-- | Inserts the string literally - no escaping, no quoting.
+literal :: String -> Expr a
+literal x = Expr (ConstExpr (OtherLit x))
+
+-- | Takes a default value a and a nullable value. If the value is NULL,
+--   the default value is returned, otherwise the value itself is returned.
+--   Simliar to 'fromMaybe'
+fromNull :: Expr a         -- ^ Default value (to be returned for 'Nothing')
+	 -> Expr (Maybe a) -- ^ A nullable expression
+	 -> Expr a
+fromNull d x@(Expr px) = _case [(isNull x, d)] (Expr px)
+
+-- | Similar to fromNull, but takes a 
+-- value argument rather than an Expr.
+fromVal :: ShowConstant a => a 
+        -> Expr (Maybe a)
+        -> Expr a
+fromVal = fromNull . constant 
+
 -- | Turn constant data into a nullable expression. 
 --   Same as @constant . Just@
-constJust :: ShowConstant a => a -> Expr (Maybe a)
-constJust x = constant (Just x)
+constExpr :: Expr a -> Expr (Maybe a)
+constExpr (Expr x) = (Expr x)
 
+-- | Turn constant data into a nullable expression. 
+--   Same as @constant . Just@
+constVal :: ShowConstant a => a -> Expr (Maybe a)
+constVal x = constant (Just x)
+
+-- | Represents a null value.
+constNull :: Expr (Maybe a)
+constNull = Expr (ConstExpr NullLit)
+
+-- | Generates a 'CAST' expression for the given
+-- expression, using the argument given as the destination
+-- type. 
+cast :: String -- ^ Destination type.
+  -> Expr a -- ^ Source expression.
+  -> Expr b
+cast typ (Expr expr) = Expr (CastExpr typ expr)
+
+-- | Coerce the type of an expression
+-- to another type. Does not affect the actual
+-- primitive value - only the `phantom' type.
+coerce :: Expr a -- ^ Source expression
+  -> Expr b -- ^ Destination type.
+coerce (Expr e) = Expr e
+
+-- | Converts records w/o Expr (usually from database
+-- queries) to records with Expr types. 
 class ConstantRecord r cr | r -> cr where
     constantRecord :: r -> cr
 
@@ -491,46 +679,14 @@ instance ConstantRecord r cr => ConstantRecord (Record r) (Record cr) where
 instance ConstantRecord RecNil RecNil where
     constantRecord RecNil = RecNil
 
-instance (ShowConstant a, ConstantRecord r cr) 
+instance (ShowConstant a, ConstantRecord r cr)
     => ConstantRecord (RecCons f a r) (RecCons f (Expr a) cr) where
     constantRecord ~(RecCons x rs) = RecCons (constant x) (constantRecord rs)
 
 -----------------------------------------------------------
 -- Aggregate operators
---
--- I have changed these to take an expression instead of
--- a relation and an attribute, since that seemed 
--- unneccessarily restrictive. I have probably overlooked 
--- something in doing so, so I left the old code commented out.
--- Bjorn Bringert, 2004-01-10
 -----------------------------------------------------------
 
-{-
-aggregate :: HasField f r => AggrOp -> Rel r -> Attr f a -> Expr b
-aggregate op rel attr
-		= Expr (AggrExpr op primExpr)
-		where
- 	  	  (Expr primExpr)  = rel ! attr
-
-count :: HasField f r => Rel r -> Attr f a -> Expr Int
-count x		= aggregate AggrCount x
-
-
-numAggregate :: (Num a,HasField f r) => AggrOp -> Rel r -> Attr f a -> Expr a
-numAggregate	= aggregate
-
-_sum,_max,_min,avg,stddev,stddevP,variance,varianceP 
-    :: (Num a,HasField f r) => Rel r -> Attr f a -> Expr a
-_sum x          = numAggregate AggrSum x
-_max x          = numAggregate AggrMax x
-_min x          = numAggregate AggrMin x
-avg x           = numAggregate AggrAvg x
-stddev x        = numAggregate AggrStdDev x
-stddevP x       = numAggregate AggrStdDevP x
-variance x      = numAggregate AggrVar x
-varianceP x     = numAggregate AggrVarP x
--}
-
 aggregate :: AggrOp -> Expr a -> ExprAggr b
 aggregate op (Expr primExpr) = ExprAggr (AggrExpr op primExpr)
 
@@ -589,14 +745,14 @@ orderOp op rel attr = OrderExpr op expr
 -- Takes a relation and an attribute of that relation, which
 -- is used for the ordering.
 asc :: HasField f r => Rel r -> Attr f a -> OrderExpr
-asc rel attr	= orderOp OpAsc rel attr
+asc rel attr = orderOp OpAsc rel attr
 
 -- | Use this together with the function 'order' to 
 -- order the results of a query in descending order.
 -- Takes a relation and an attribute of that relation, which
 -- is used for the ordering.
-desc :: HasField f r => Rel r -> Attr f a -> OrderExpr
-desc rel attr	= orderOp OpDesc rel attr
+desc :: (HasField f r) => Rel r -> Attr f a -> OrderExpr
+desc rel attr = orderOp OpDesc rel attr
 
 -- | Order the results of a query.
 -- Use this with the 'asc' or 'desc' functions.
@@ -607,6 +763,9 @@ order xs = updatePrimQuery_ (Special (Order xs))
 -- Query Monad
 -----------------------------------------------------------
 
+unQuery :: Query a -> a
+unQuery (Query g) = fst $ g (1, Empty)
+
 runQuery :: Query (Rel r) -> PrimQuery
 runQuery = fst . runQueryRel
 
@@ -616,7 +775,24 @@ runQueryRel (Query f)
               assoc   = zip scheme (map (AttrExpr . fresh alias) scheme)
           in  (Project assoc primQuery, Rel 0 scheme)
 
-
+-- | Allows a subquery to be created between another query and
+-- this query. Normally query definition is associative and query definition
+-- is interleaved. This combinator ensures the given query is
+-- added as a whole piece.
+subQuery :: Query (Rel r) -> Query (Rel r)
+subQuery (Query qs) = Query make
+  where
+    make (currentAlias, currentQry) =
+        -- Take the query to add and run it first, using the current alias as
+        -- a seed.
+        let (Rel otherAlias otherScheme,(newestAlias, otherQuery)) = qs (currentAlias,Empty)
+            -- Effectively renames all columns in otherQuery to make them unique in this
+            -- query.
+            assoc = zip (map (fresh newestAlias) otherScheme)
+                        (map (AttrExpr . fresh otherAlias) otherScheme)
+            -- Produce a query which is a cross product of the other query and the current query.
+        in (Rel newestAlias otherScheme, (newestAlias + 1, times (Project assoc otherQuery) currentQry))
+            
 instance Functor Query where
   fmap f (Query g)      = Query (\q0 -> let (x,q1) = g q0  in (f x,q1))
 
@@ -635,7 +811,6 @@ updatePrimQuery_ f = updatePrimQuery f >> return ()
 newAlias :: Query Alias
 newAlias                = Query (\(i,qt) -> (i,(i+1,qt)))
 
-
 -- fresh 0 is used in the 'Database' module
 fresh :: Alias -> Attribute -> Attribute
 fresh 0     attribute   = attribute
@@ -658,12 +833,3 @@ instance ToPrimExprs RecNil where
 instance (ExprC e, ToPrimExprs r) => ToPrimExprs (RecCons l (e a) r) where
     toPrimExprs ~(RecCons e r) = primExpr e : toPrimExprs r
 
-{-
-
-exprs :: ShowRecRow r => Record r -> [PrimExpr]
-exprs r         = map (readPrimExpr . snd) (showRecRow r)
-                where
-                  readPrimExpr s   = case (reads (s "")) of
-                                    [(Expr qx,_)] -> qx
-                                    _             -> error ("record with invalid expression value: " ++ (s ""))
--}
