@@ -38,7 +38,7 @@ ppSql (SqlSelect options attrs tables criteria groupby orderby extra)
       <+> ppAttrs attrs
       $$ ppTables tables
       $$ ppWhere criteria
-      $$ ppGroupBy groupby
+      $$ maybe empty ppGroupBy groupby
       $$ ppOrderBy orderby
       $$ hsep (map text extra)
 ppSql (SqlBin op q1 q2) = parens (ppSql q1) $$ text op $$ parens (ppSql q2)
@@ -73,14 +73,14 @@ ppWhere [] = empty
 ppWhere es = text "WHERE" 
              <+> hsep (intersperse (text "AND") (map ppSqlExpr es))
 
-ppGroupBy :: [(SqlColumn, SqlExpr)] -> Doc
-ppGroupBy [] = empty
-ppGroupBy es = text "GROUP BY" <+> ppGroupAttrs es
+ppGroupBy :: Mark -> Doc
+ppGroupBy All = error "Should not ever print GroupBy all."
+ppGroupBy (Columns es) = text "GROUP BY" <+> ppGroupAttrs es
   where
     ppGroupAttrs :: [(SqlColumn, SqlExpr)] -> Doc
     ppGroupAttrs cs = commaV nameOrExpr cs
     nameOrExpr :: (SqlColumn, SqlExpr) -> Doc
-    nameOrExpr (name, ColumnSqlExpr _) = text name
+    nameOrExpr (_, ColumnSqlExpr col) = text col
     nameOrExpr (_, expr) = parens (ppSqlExpr expr)
     
 ppOrderBy :: [(SqlExpr,SqlOrder)] -> Doc
@@ -167,10 +167,12 @@ ppSqlExpr :: SqlExpr -> Doc
 ppSqlExpr e =
     case e of
       ColumnSqlExpr c     -> text c
-      BinSqlExpr op e1 e2 -> ppSqlExpr e1 <+> text op <+> ppSqlExpr e2
+      ParensSqlExpr e -> parens (ppSqlExpr e)
+      BinSqlExpr op e1 e2 -> ppSqlExpr e1 <+> text op <+> ppSqlExpr e2 
       PrefixSqlExpr op e  -> text op <+> ppSqlExpr e
       PostfixSqlExpr op e -> ppSqlExpr e <+> text op
       FunSqlExpr f es     -> text f <> parens (commaH ppSqlExpr es)
+      AggrFunSqlExpr f es     -> text f <> parens (commaH ppSqlExpr es)
       ConstSqlExpr c      -> text c
       CaseSqlExpr cs el   -> text "CASE" <+> vcat (map ppWhen cs)
                              <+> text "ELSE" <+> ppSqlExpr el <+> text "END"
@@ -178,6 +180,10 @@ ppSqlExpr e =
                                <+> text "THEN" <+> ppSqlExpr t
       ListSqlExpr es      -> parens (commaH ppSqlExpr es)
       ExistsSqlExpr s     -> text "EXISTS" <+> parens (ppSql s)
+      ParamSqlExpr n v -> ppSqlExpr v
+      PlaceHolderSqlExpr -> text "?"
+      CastSqlExpr typ expr -> text "CAST" <> parens (ppSqlExpr expr <+> text "AS" <+> text typ)
+    
 
 commaH :: (a -> Doc) -> [a] -> Doc
 commaH f = hcat . punctuate comma . map f
